package org.prelle.audiotron;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Created at 24.04.2004, 19:47:06
 *
 * @author Stefan Prelle
 *
 */
public class AudioFrame extends Frame implements ActionListener, ItemListener {
	
	class Track {
		String title;
		String file;
		int volume;
		public Track(String title, String file, int vol) {
			this.title=title;
			this.file=file;
			this.volume=vol;
		}
		public String toString() {
			return title;
		}
	}
	
	private RequestQueue atron;
	private CommandRequest command;
	private QueueRequest queue;
	
	private Track[] tracks;
	private Choice choice, volume;
	private Checkbox repeat;
	private Button play, stop;
	
	//--------------------------------------------------------------
	public static void main(String[] args) {
	  if (args.length!=4) {
	    System.err.println("Usage: java AudioFrame <host> <user> <password> <playlist>");
	    System.exit(1);
	  }
	    
		String host = args[0];
		String user = args[1];
		String pass = args[2];
		String list = args[3];
		
		try {
			AudioFrame aframe = new AudioFrame(host, user, pass, list);
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//--------------------------------------------------------------
	public AudioFrame(String host, String user, String pass, String list) throws UnknownHostException, IOException {
		super("AudioTron Control");
		Socket sock = new Socket(host,80);
		System.out.println("Connected");
		atron = new RequestQueue(sock,user,pass);
		atron.add(new CommandRequest(CommandRequest.POWERON));
		atron.add(new CommandRequest(CommandRequest.CLEAR));
		atron.add(new CommandRequest(CommandRequest.STOP));
		
		parsePlaylist(list);
		initGUI();
		clearRepeat();
	}
  
	//-------------------------------------------------------------------------
	private void parsePlaylist(String list) throws IOException {
		File listFile = new File(list);
		System.out.println("Parsing playlist: "+listFile.getAbsolutePath());
		BufferedReader in = new BufferedReader(new FileReader(listFile));
		
	  Vector v = new Vector();
		while (in.ready()) {
			String line = in.readLine();
			if (line.trim().length()==0)
				continue;
			StringTokenizer tok = new StringTokenizer(line);
			String file = tok.nextToken();
			int vol = -50;
			if (tok.hasMoreTokens()) {
			  try {
			    vol = Integer.parseInt(tok.nextToken());
			  } catch (NumberFormatException nfe) {
			    System.out.println("Invalid volume in line: "+line);
			  }
			}
			String title = file.substring(file.lastIndexOf("\\")+1,file.length());
			if (title.length()>30)
				title = title.substring(0, 30);
			v.addElement(new Track(title,file,vol));
//			queue = new QueueRequest(QueueRequest.FILE, file);
//			atron.add(queue);
		}
		
		tracks = new Track[v.size()];
		Enumeration e = v.elements();
		int i=0;
		while (e.hasMoreElements()) {
			tracks[i++] = (Track) e.nextElement();
		}
	}
	
	//-------------------------------------------------------------------------
	private void clearRepeat() {
		atron.add(new CommandRequest(CommandRequest.REPEAT,CommandRequest.STATE_OFF));
//		atron.add(new CommandRequest(CommandRequest.CLEAR));
//		for (int i=0; i<tracks.length; i++)
//			atron.add(new QueueRequest(QueueRequest.FILE, tracks[i].file));
//		atron.add(new CommandRequest(CommandRequest.PLAY));
//  	atron.add(new InfoRequest(InfoRequest.PLAYQ));
	}
	
	//-------------------------------------------------------------------------
//	private void setRepeat(Track track) {
//		atron.add(new CommandRequest(CommandRequest.REPEAT,CommandRequest.STATE_ON));
//		atron.add(new CommandRequest(CommandRequest.CLEAR));
//		atron.add(new QueueRequest(QueueRequest.FILE, track.file));
//		atron.add(new CommandRequest(CommandRequest.PLAY));
//		atron.add(new CommandRequest(CommandRequest.GOTO,0));
//  	atron.add(new InfoRequest(InfoRequest.PLAYQ));
//
//	}
	
	//-------------------------------------------------------------------------
	/** Plaziert eine Komponente in einem GridBag-Layoutmanager */
	//-------------------------------------------------------------------------
	private void place(Component co, int x, int y, int w, int h, int a) {
		GridBagConstraints c = new GridBagConstraints();
		c.gridx     = x;
		c.gridy     = y;
		c.gridwidth = w;
		c.gridheight= h;
		c.fill      = GridBagConstraints.NONE;
		c.anchor    = a;
		c.weightx   = 0.0;
		c.weighty   = 0.0;
		c.insets = new Insets(3,3,3,3);
		((GridBagLayout)this.getLayout()).setConstraints(co, c);
		this.add(co);
	}
	
	//--------------------------------------------------------------
	private void initGUI() {
		this.setLayout(new GridBagLayout());
		this.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent arg0) {
			}
			public void windowClosed(WindowEvent arg0) {
			}
			public void windowClosing(WindowEvent arg0) {
				command = new CommandRequest(CommandRequest.POWEROFF);
				atron.add(command);
				System.exit(0);
			}
			public void windowDeactivated(WindowEvent arg0) {
			}
			public void windowDeiconified(WindowEvent arg0) {
			}
			public void windowIconified(WindowEvent arg0) {
			}
			public void windowOpened(WindowEvent arg0) {
			}
		});
		
		Label tracks_l = new Label("Tracks");
		choice = new Choice();
		for (int i=0; i<tracks.length; i++)
			choice.add(tracks[i].toString());
		
		Label vol_l = new Label("Volume");
		volume = new Choice();
		for (int i=0; i<10; i++)
			volume.add((-2*i)+" dB");
		volume.addItemListener(this);
		
		repeat = new Checkbox("Repeat");
		repeat.addItemListener(this);
		
		play = new Button("Play");
		play.addActionListener(this);
		stop = new Button("Stop");
		stop.addActionListener(this);
		
		this.place(tracks_l, 0,0, 2,1, GridBagConstraints.WEST);
		this.place(choice  , 0,1, 2,1, GridBagConstraints.WEST);
		this.place(vol_l   , 0,2, 1,1, GridBagConstraints.WEST);
		this.place(volume  , 1,2, 1,1, GridBagConstraints.WEST);
		this.place(repeat  , 0,3, 2,1, GridBagConstraints.WEST);
		this.place(play    , 0,4, 1,1, GridBagConstraints.EAST);
		this.place(stop    , 1,4, 1,1, GridBagConstraints.WEST);
		
		this.pack();
		this.show();
	}

	//------------------------------------------------
	/* 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource()==play) {
			System.out.println("Now starting "+choice.getSelectedItem());
			int index = choice.getSelectedIndex();
			atron.add(new CommandRequest(CommandRequest.CLEAR));
			atron.add(new QueueRequest(QueueRequest.FILE, tracks[index].file));
			atron.add(new CommandRequest(CommandRequest.PLAY));
			atron.add(new CommandRequest(CommandRequest.VOLUME, tracks[index].volume));
			atron.add(new CommandRequest(CommandRequest.GOTO, 0));
//	  	atron.add(new InfoRequest(InfoRequest.PLAYQ));
		} else if (arg0.getSource()==stop) {
			command = new CommandRequest(CommandRequest.STOP);
			atron.add(command);			
		}
	}

	//------------------------------------------------
	/* 
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent arg0) {
		if (arg0.getSource()==repeat) {
			boolean state = repeat.getState();
			atron.add(new CommandRequest(CommandRequest.REPEAT,state?CommandRequest.STATE_ON:CommandRequest.STATE_OFF));
//	  	atron.add(new InfoRequest(InfoRequest.PLAYQ));

//			if (repeat.getState())
//				setRepeat(tracks[choice.getSelectedIndex()]);
//			else
//				clearRepeat();
		} else if (arg0.getSource()==volume) {
		  int vol = -2*volume.getSelectedIndex();
			atron.add(new CommandRequest(CommandRequest.VOLUME, vol));
		}
	}
	
	
}
