package org.prelle.audiotron;

/**
 * Created at 23.03.2004, 12:36:44
 *
 * @author Stefan Prelle
 *
 */
public class CommandRequest extends Request {
	
	private final static String URL = "apicmd.asp?cmd=";
	
	// Simple commands
	public final static int PLAY     = 0;
	public final static int STOP     = 1;
	public final static int NEXT     = 2;
	public final static int PREV     = 3;
	public final static int CLEAR    = 4;
	public final static int POWERON  = 5;
	public final static int POWEROFF = 6;
	public final static int MSGOFF   = 7;
	// State commands
	public final static int PAUSE    = 8;
	public final static int RANDOM   = 9;
	public final static int REPEAT   = 10;
	public final static int MUTE     = 11;
	public final static int GOTO     = 12;
	public final static int VOLUME   = 13;
	public final static int POSITION = 14;
	
	// States
	public final static int STATE_ON     = 1;
	public final static int STATE_OFF    = -1;
	public final static int STATE_TOGGLE = 0;
	
	private int type;
	private int arg;
	
	//------------------------------------------------
	public CommandRequest(int command) {
		if (command<PLAY || command>MSGOFF)
			throw new IllegalArgumentException("Invalid command ID for simple commands");
		type = command;
	}
	
	//------------------------------------------------
	public CommandRequest(int command, int arg) {
		if (command<PAUSE)
			throw new IllegalArgumentException("Invalid command ID for state commands");
		type = command;
		this.arg = arg;
	}
		
	//------------------------------------------------
	/* 
	 * @see org.prelle.audiotron.Request#getRequestURI()
	 */
	public String getRequestURI() {
		switch(type) {
	  case PLAY    : return URL+"play";	
	  case STOP    : return URL+"stop";	
	  case NEXT    : return URL+"next";	
	  case PREV    : return URL+"stop";	
	  case CLEAR   : return URL+"clear";	
	  case POWERON : return URL+"poweron";	
	  case POWEROFF: return URL+"poweroff";	
	  case PAUSE   : return URL+"pause&arg="+arg;	
	  case RANDOM  : return URL+"random&arg="+arg;	
	  case REPEAT  : return URL+"repeat&arg="+arg;	
	  case MUTE    : return URL+"mute&arg="+arg;	
	  case GOTO    : return URL+"goto&arg="+arg;	
	  case VOLUME  : return URL+"volume&arg="+arg;	
	  case POSITION: return URL+"position&arg="+arg;	
	  default:
	  	return null;
		}
	}
	//------------------------------------------------
	/* 
	 * @see org.prelle.audiotron.Request#processResult(java.lang.String)
	 */
	public void processResult(String result) {
		// TODO Auto-generated method stub
	}
}
