package org.prelle.audiotron;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.Socket;
import java.net.URL;
import java.util.ArrayList;

/**
 * A RequestQueue handles HTTP communication with a single device.
 * 
 * Created at 23.03.2004, 12:31:32
 *
 * @author Stefan Prelle
 *
 */
public class RequestQueue implements Runnable {
	
	private ArrayList queue;
	private String urlPrefix;
	private String authToken;
	private PrintWriter out;
	private BufferedReader in;
	private Thread thread;
	
	//------------------------------------------------
	public RequestQueue(Socket sock, String login, String passwd) throws IOException {
		urlPrefix = "http://"+sock.getInetAddress().getHostName()+"/";

    // Authentication
    String foo = login+":"+passwd;
    authToken = "Basic "+new sun.misc.BASE64Encoder().encode(foo.getBytes());
		
		out = new PrintWriter(new OutputStreamWriter(sock.getOutputStream()));
		in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
		queue = new ArrayList();
		thread = new Thread(this, "RequestQueue "+sock.getInetAddress().getHostName());
		thread.start();
//		thread.setDaemon(true);
	}

	//------------------------------------------------
	/* 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		System.out.println("Started queue");
		try {
			while (true) {
				Request request = null;
				synchronized (queue) {
					// Wait for items in the queue
					while (queue.size()==0)
						try {
							queue.wait();
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						request = (Request)queue.get(0);
						queue.remove(request);
				}
				
				URL url = new URL(urlPrefix+request.getRequestURI());
				System.out.println("Requesting "+url);
				HttpURLConnection con = (HttpURLConnection)url.openConnection();
		    con.setRequestProperty("User-Agent", "libaudiotron");
		    // Authentication
		    con.setRequestProperty("Authorization", authToken);
		    System.out.println(con.getResponseCode());
		    System.out.println(con.getResponseMessage());
//		    
//				System.out.println("GET "+url);
//				out.println("GET "+url);
//				out.flush();
//				// Await reply
				parseReply(request, con.getInputStream());
				System.out.println("--> Reply parsed");
			}
		} catch (IOException ioe) {
		  ioe.printStackTrace();
			System.err.println("RequestQueue died: "+ioe);
		}
	}

		//------------------------------------------------
		private void parseReply(Request request, InputStream is) throws IOException {
			BufferedReader in = new BufferedReader(new InputStreamReader(is));
			// Parse message header
			while (true) {
				String line = in.readLine();
				System.out.println(">"+line);
				if (line.length()==0)
					break;
			}
		}

		//------------------------------------------------
		/**
		 * Add a request to the queue.
		 * @param request Request to add
		 * @return Queue-size after adding the request
		 */
		public int add(Request request) {
			synchronized (queue) {
				queue.add(request);
				queue.notify();
				return queue.size();
			}
		}
		
}
